/*

   Copyright 2010, 2011 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package lf.api;

import java.util.LinkedList;
import java.util.ListIterator;

import cf.api.ApplicationKnowledge;

public class AppKnowledgeBase extends LinkedList<ApplicationKnowledge>
{
  public synchronized void update(ApplicationKnowledge k)
  {
    remove(k);
    add(k);
    additions = true;
  }

  public synchronized void remove(ApplicationKnowledge k)
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      //SimEngine.log.println(" - existing local knowledge: ");
      //SimEngine.log.println(" - " + e.logRepLong());
      //SimEngine.log.println(" - " + k.logRepLong());
      // SimEngine.log.println(" - related: " + e.related(k));
      // SimEngine.log.println(" - replaces: " + k.replaces(e));

      //if(e.related(k) && k.subsumed(e) && !e.subsumed(k))
      if(e.related(k) && k.replaces(e))
      { 
        //SimEngine.log.println(" - removing local knowledge: ");
        //SimEngine.log.println(" - " + e.logRepLong());
        li.remove(); 
      }
    }
  }

  //public ApplicationKnowledge subsumed(ApplicationKnowledge k)
  public synchronized ApplicationKnowledge ignore(ApplicationKnowledge k)
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();

      //if(e.related(k) && e.subsumed(k)) 
      if(e.related(k) && (e.subsumes(k) || e.replaces(k))) 
      {
        return e;
      }
    }
    return null;
  }

  public synchronized void removeExpiredKnowledge(ApplicationKnowledge k)
  {
    //if(k.expired()) remove(k);
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      if(e.related(k) && e.expired())
      {
        //System.out.println(e.logRepLong());
        li.remove();
      }
    }
  }

  public synchronized void removeExpiredKnowledge()
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      if(e.expired()) li.remove();
    }
  }

  boolean additions = false;

  public void done()
  {
    additions = false;
  }

  public boolean additions()
  {
    return additions;
  }

  public boolean fresh(ApplicationKnowledge k)
  {
    if(ignore(k) == null)
      return true;
    else 
      return false;
  }

  void print()
  {
    for(ApplicationKnowledge k : this) 
    {
      //Simulation.rep.println(k.logRepLong());
      k.print();
    }
  }

}
