/*

   Copyright 2010, 2011 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package lf.api;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApplication;
import cf.sim.SimEngine;

// Knowledge about Facts

public class Fact extends ApplicationKnowledge
{
    String robot;

    /** Uniquely determines fact. */

    String Term;
	FactType type; 

    /** Observes. */

    public String term() { return Term; }
    public String robot() { return robot; }
	public FactType type() { return type; }
	public void setrobot(String name) { robot = name; } 

    /** Constructor. */

    public Fact(CyberApplication creator, String tr)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.Term = tr;
		this.type = FactType.Undefined;
    }

    public Fact(CyberApplication creator, String tr, FactType type)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.Term = tr;
		this.type = type; 
    }

    public boolean related(ApplicationKnowledge k)
    {
		//log("def related ordering - " + this.toString() + " is related to " + k.toString());

        if (k instanceof Fact)
        {
            Fact f = (Fact) k;

           	return term().equals(f.term());
        }      
    	
		return false;
    }

    public boolean subsumes(ApplicationKnowledge k)
	{
		if (k instanceof Fact)
		{
          	Fact f = (Fact) k;

			if (term().equals(f.term()))
			{
				//log(toString() + " subsumes " + f.toString());
				return true; 
			}
			else 
				return false;
		}

		return false;
	}

    public boolean replaces(ApplicationKnowledge k)
    {
		//log("def replace ordering - " + this.toString() + " replaces " + k.toString());

		if (k instanceof Fact)
		{
    		Fact f = (Fact)k;

			if (term().equals(f.term()) && (f.creationTime() < creationTime())) 
			{
				//log(term().toString() + " replaces " + f.term().toString());
        		return true;
			}
		}
		
		return false; 
    }

    public String toString()
    {
        return "Fact(" + creationTime() + ", " + creator() + ", TR(" + Term + ")"; 
    }

    public String logRepLong()
    {
        return toString();
    }

    public void log(String s)
    {
        SimEngine.log.println(s); 
    }
}

