/*

   Copyright 2010, 2011 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package lf.api;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApplication;
import cf.sim.SimEngine;
import alice.tuprolog.Struct;
import alice.tuprolog.Term;

public class Goal extends ApplicationKnowledge   
{
    /** Uniquely determines goal. */

    private String logicalFormula;
    private Boolean satisfied;
	private GoalType type;

    /** Observes. */

    public String lf() { return logicalFormula; }
    public boolean satisfied() { return satisfied; }
    public GoalType type() { return type; }

    /** Constructor. */

    public Goal(CyberApplication creator, String lf)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.logicalFormula = lf;
        this.satisfied = false;
		this.type = GoalType.Undefined;
    }

    public Goal(CyberApplication creator, String lf, java.lang.Double timeToExpire)
    {
        super(creator, timeToExpire);
        this.logicalFormula = lf;
        this.satisfied = false;
		this.type = GoalType.Undefined;
    }

    public Goal(CyberApplication creator, String lf, boolean s)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.logicalFormula = lf;
        this.satisfied = s;
		this.type = GoalType.Undefined;
    }

    public Goal(CyberApplication creator, String lf, GoalType type)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.logicalFormula = lf;
        this.satisfied = false;
		this.type = type;
    }

    public Goal(CyberApplication creator, String lf, java.lang.Double timeToExpire, GoalType type)
    {
        super(creator, timeToExpire);
        this.logicalFormula = lf;
        this.satisfied = false;
		this.type = type;
    }

    public Goal(CyberApplication creator, String lf, boolean s, GoalType type)
    {
        super(creator, java.lang.Double.POSITIVE_INFINITY);
        this.logicalFormula = lf;
        this.satisfied = s;
		this.type = type;
    }

    public boolean related(ApplicationKnowledge k)
    {
		//log("def related ordering - " + this.toString() + " is related to " + k.toString());

		if (k instanceof Goal)
		{
        	Goal g = (Goal) k;

            Term t = Term.createTerm(g.lf().substring(0,g.lf().length()-1));
       		Term my_t = Term.createTerm(lf().substring(0,lf().length()-1));

        	if (t.equals(my_t)) return true;

        	if (t.match(my_t))
        	{
            	Struct st = (Struct) t;
        		Struct my_st = (Struct) my_t;

            	int arity = st.getArity();
            	for (int i=0; i < arity; i++)
            	{
                	Term arg1 = st.getArg(i);
                	Term arg2 = my_st.getArg(i);
                	if (!arg1.isVar() && arg2.isVar()) return false;
                	if (arg1.isVar() && !arg2.isVar()) return false;
            	}
            	return true;
        	}
    	}

		return false; 
	}

    public boolean subsumes(ApplicationKnowledge k)
	{
		if (k instanceof Goal)
		{
        	Goal g = (Goal) k;

			Term t = Term.createTerm(g.lf().substring(0,g.lf().length()-1));
        	Term my_t = Term.createTerm(lf().substring(0,lf().length()-1));

			if (my_t.equals(t))
			{
				//log(my_t.toString() + " subsumes " + t.toString());
				return true; 
			}
			else 
				return false; 
		}

		return false;
	}

    public boolean replaces(ApplicationKnowledge k)
    {
		//log("def replace ordering - " + this.toString() + " replaces " + k.toString());

		if (k instanceof Goal)
		{
        	Goal g = (Goal) k;

			Term t = Term.createTerm(g.lf().substring(0,g.lf().length()-1));
        	Term my_t = Term.createTerm(lf().substring(0,lf().length()-1));

        	if (t.match(my_t) && (g.creationTime() < creationTime())) 
			{
				//log(my_t.toString() + " replaces " + t.toString());
				return true; 
			}
		}

		return false;
	}

    public String toString()
    {
        return "Goal(" + creationTime() + ", " + creator() + ", LF(" + lf() +
")" +
            (satisfied() ? " satisfied " : " ");
    }

    public String logRepLong()
    {
        return toString();
    }

    public void log(String s)
    {
        SimEngine.log.println(s);
    }

} 

